import datetime
import logging
import pipes
import subprocess
import time

import target

class Screenshotter(object):
    """
    Implements all screenshot-related logic.
    """

    def __init__(self, config):
        self.config = config
        self.screenshot_filenames = []

    def take_screenshots(self):
        """
        Runs through the process of taking all screenshots.
        """
        targets = []
        for target_config in self.config.get('targets'):
            current_target = target.Target(self.config, target_config)
            try:
                self.depict(
                    current_target.url,
                    current_target.selector,
                    current_target.local_filepath,
                    # Depict's delay argument is defined in milliseconds
                    str(int(current_target.page_load_delay) * 1000)
                )
                targets.append(current_target)
            except RuntimeError as e:
                logging.error(e)
        return targets

    def depict(self, url, selector, destination, page_load_delay):
        """
        Runs the command-line utility `depict`.
        """
        args = ['depict', url, destination, '-s', selector,
                '--delay', page_load_delay]

        override_css_file = self.config.get('override_css_file')
        if override_css_file:
            args = args + ['--css', override_css_file]

        if self.config.get('wait_for_js_signal', False):
            args = args + ['--call-phantom']

        # Quote each argument and combine them into a single string
        args_string = ''
        for arg in args:
            quoted_arg = pipes.quote(arg)
            args_string += ' ' + quoted_arg

        logging.info('Running shell command: %s' % (args_string))

        # Use shell=True because selectors may contain characters that would
        # otherwise be stripped out by subprocess.Popen() (e.g. `#`, `<`, etc.
        p = subprocess.Popen(args_string, shell=True, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)

        # Wait at most `failure_timeout` seconds for the process to finish
        failure_timeout = self.config.get('failure_timeout', 30)
        deadline = time.time() + failure_timeout
        while time.time() < deadline and p.poll() == None:
            time.sleep(1)

        # If the failure timeout is more than 0 and the process still hasn't
        # finished, terminate it.
        if failure_timeout > 0 and p.poll() == None:
            p.terminate()
            raise RuntimeError('Terminated shell command: %s' % (args))

        out, err = p.communicate()
        if p.returncode != 0:
            raise RuntimeError('depict error: %s' % err)



import logging

from boto.s3.connection import Key, S3Connection

class Uploader(object):
    """
    Implements all upload-related logic.
    """

    def __init__(self, config):
        self.config = config
        conn = S3Connection(self.config.get('aws_access_key'),
                self.config.get('aws_secret_key'))

        # The bucket must already exist.
        self.bucket = conn.get_bucket(self.config.get('aws_bucket'))

    def upload_screenshots(self, images):
        """
        Runs through the process of uploading all screenshots.

        Uploads each image to both its filepath and the "latest" filepath.
        """
        filepaths = []
        for image in images:
            filepaths.append(self.upload(image.local_filepath,
                    image.aws_filepath))
            logging.info('Sucessfully uploaded image to %s' % image.public_url)

            filepaths.append(self.upload(image.local_filepath,
                    image.aws_latest_filepath))
            logging.info('Sucessfully uploaded image to %s'
                    % image.public_latest_url)
        return filepaths

    def upload(self, local_filepath, aws_filepath):
        """
        Uploads `local_filepath` to `aws_filepath`.

        Returns the published URL for the file.
        """
        logging.info('Publishing %s to %s' % (
                local_filepath, aws_filepath))

        key = Key(bucket=self.bucket, name=aws_filepath)
        key.key = aws_filepath
        key.set_contents_from_filename(local_filepath)
        key.set_acl('public-read')



#!/usr/bin/env python

import logging
import os
import subprocess
import sys
import time

import yaml

import screenshotter
import uploader

class WhipperSnapper(object):
    """
    Implements all screenshot-related logic.
    """

    def __init__(self):
        if len(sys.argv) != 2:
            self.usage()
            sys.exit(1)

        config_filepath = sys.argv[1]
        self.config = self.load_config(config_filepath)
        self.log_file = self.init_logging()
        self.screenshotter = screenshotter.Screenshotter(self.config)
        if not self.config.get('skip_upload'):
            self.uploader = uploader.Uploader(self.config)

    def main(self):
        """
        Runs through the full screenshot process.
        """

        print """
Whippersnapper is running. To view its log file:

    tail -f %s

To quit, press ^C (ctrl-C).""" % (self.log_file)

        while True:
            targets = self.screenshotter.take_screenshots()
            self.uploader.upload_screenshots(targets)
            # TODO Image delete code probably doesn't belong here
            if (self.config.get('delete_local_images')):
                [os.remove(path.get('local_filepath')) for path in targets]
            time.sleep(self.config.get('time_between_screenshots'))

    def init_logging(self):
        """
        Create a log file, and attach a basic logger to it.
        """
        log_file = self.config.get('log_file')
        # Create the log file if it does not yet exist
        with open(log_file, 'a+'):
            pass
        logging.basicConfig(filename=log_file,
                format='%(levelname)s:%(asctime)s %(message)s',
                level=logging.INFO)
        return log_file

    def load_config(self, config_filepath):
        """
        Load configuration from config.yaml.

        Many options have defaults; use these unless they are overwritten in
        config.yaml. This file includes the urls, css selectors and slugs for
        the targets to screenshot.
        """

        log_file = os.path.abspath(os.path.expanduser(os.path.dirname(
                os.path.abspath(__file__)) + '/../whippersnapper.log'))

        config = {
            'skip_upload': False,
            'aws_bucket': '',
            'aws_subpath': '',
            'aws_access_key': None,
            'aws_secret_key': None,
            'log_file': log_file,
            'delete_local_images': False,
            'time_between_screenshots': 60,
            'override_css_file': None,
            'page_load_delay': 2,
            'wait_for_js_signal': False,
            'failure_timeout': 30,
        }

        required = (
            'targets',
            'local_image_directory',
        )

        raw_config = None

        with open(config_filepath) as f:
            raw_config = yaml.load(f)

        for option_name, option_value in raw_config.iteritems():
            config[option_name] = option_value

        for option in required:
            try:
                config[option] = raw_config[option]
            except KeyError:
                raise RuntimeError('Config is missing required attribute: %s'
                        % option)

        return config

    def usage(self):
        """
        Print usage information.
        """
        print """
        USAGE: whippersnapper CONFIG_FILEPATH
        """

def launch_new_instance():
    """
    Launch an instance of Whippersnapper.
    """
    try:
        s = WhipperSnapper()
        s.main()
    except KeyboardInterrupt:
        # Print a blank line
        print

if __name__ == '__main__':
    launch_new_instance()



#!/usr/bin/env python

from setuptools import setup

setup(
    name='whippersnapper',
    version='0.0.0',
    author='Kevin Schaul, Katie Park',
    author_email='kevin.schaul@washpost.com, katie.park@washpost.com',
    url='http://github.com/washingtonpost/whippersnapper',
    description='Whippersnapper is an automated screenshot tool to keep a visual history of content on the web.',
    long_description='Check out the project on GitHub for the latest information <http://github.com/washingtonpost/whippersnapper>',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python',
    ],
    packages=[
        'whippersnapper',
    ],
    entry_points = {
        'console_scripts': [
            'whippersnapper = whippersnapper.whippersnapper:launch_new_instance',
        ],
    },
    install_requires = [
        'PyYAML>=3.11',
        'boto>=2.32.1',
        'wsgiref>=0.1.2'
    ]
)



import datetime
import os

class Target(object):
    def __init__(self, global_config, target_config):
        self.check_config_options(target_config)
        self.combine_config_options(global_config, target_config)
        self.current_datetime_string = get_current_datetime_string()

    def check_config_options(self, target_config):
        """
        Checking for the required target options.
        """
        required_options = ['slug', 'url']

        for option in required_options:
            if not option in target_config:
                raise RuntimeError('Missing required option %s' % option)

        if not 'selector' in target_config:
            target_config['selector'] = 'body'


    def combine_config_options(self, global_config, target_config):
        """
        Creates options for the target, prioritizing target-specific
        options over global options.
        """
        options_whitelist = [
            'page_load_delay',
            'wait_for_js_signal',
            'local_image_directory',
            'aws_bucket',
            'aws_subpath',
            'override_css_file',
            'wait_for_js_signal',
            'failure_timeout',
        ]

        for option in options_whitelist:
            setattr(self, option, global_config[option])

        for option in target_config:
            setattr(self, option, target_config[option])

    @property
    def filepath(self):
        """
        Generates a filepath for the image in the following form:

        slug/YYYY-MM-DD-HHMMSS-slug.png

        Used to generate local and aws filepaths.
        """
        slug = self.slug
        return '%s/%s-%s.png' % (slug, self.current_datetime_string, slug)

    @property
    def local_filepath(self):
        """
        Generates a local filepath for the image, using
        `generate_image_filepath()`
        """
        local_image_directory = os.path.abspath(os.path.expanduser(self.local_image_directory))
        image_filepath = self.filepath
        return '%s/%s' % (local_image_directory, image_filepath)

    @property
    def aws_filepath(self):
        """
        Generates an aws filepath for the image, using
        `generate_image_filepath()`
        """
        aws_subpath = self.aws_subpath
        image_filepath = self.filepath
        return '%s/%s' % (aws_subpath, image_filepath)

    @property
    def aws_latest_filepath(self):
        """
        Generates an aws "latest" filepath for the image.
        """
        aws_subpath = self.aws_subpath
        slug = self.slug
        return '%s/%s/latest-%s.png' % (aws_subpath, slug, slug)

    @property
    def public_url(self):
        """
        Generates a public URL for the file.
        """
        aws_bucket = self.aws_bucket
        aws_subpath = self.aws_subpath
        aws_filepath = self.aws_filepath
        return 'http://s3.amazonaws.com/%s/%s' % (aws_bucket, aws_filepath)

    @property
    def public_latest_url(self):
        """
        Generates a public "latest" URL for the file.
        """
        aws_bucket = self.aws_bucket
        aws_subpath = self.aws_subpath
        aws_latest_filepath = self.aws_latest_filepath
        return ('http://s3.amazonaws.com/%s/%s'
                % (aws_bucket, aws_latest_filepath))

def get_current_datetime_string():
    """
    Returns the current time in a string suitable for filenames.
    """
    now = datetime.datetime.now()
    return now.strftime('%Y-%m-%d-%H%M%S')
